#!/usr/bin/env python

import os
import json
import flask
import inspect
from . import protocookie as pc

cdir = os.path.dirname(inspect.getfile(inspect.currentframe()))

with open(os.path.join(cdir, "config.json")) as fd:
    config = json.load(fd)

pc.encryptionKey = config["encryptionKey"]

app = flask.Flask(__name__)

cookieSetterTemplate = """
    <!doctype html>
    <title>Protobuf Cookie</title>
    <form action="/">
        <button type="submit">Reload page</button>
    </form>
    {% if authdata %}
      <p>Current authentication data from cookie:</p>
      <pre>{{ authdata}}</pre>
      <p>Raw cookie value (base64 encoded encrypted data):</p>
      <pre>{{ rawauthdata}}</pre>
    {% endif %}
    <p>Setting new authentication cookie from json:</p>
    <pre>{{ jsondata}}</pre>
    <p>
    {% if authdata %}
    <form action="/del">
        <button type="submit">Remove cookie</button>
    </form>
    {% endif %}
"""

@app.route('/')
def index():
    authdata = ""
    rawauthdata = flask.request.cookies.get(config["cookieName"])
    if rawauthdata:
        authdata=pc.decrypt(rawauthdata.encode())
    resp = flask.make_response(
            flask.render_template_string(
                cookieSetterTemplate,
                authdata=authdata,
                rawauthdata=rawauthdata,
                jsondata=json.dumps(
                    config["userDefinition"],
                    sort_keys = True,
                    indent = 4,
                    separators = (',', ': '))
                )
            )
    resp.set_cookie(
            config["cookieName"],
            pc.encrypt(config["userDefinition"]))
    return resp

@app.route('/del')
def delete_cookie():
        resp = flask.make_response(flask.redirect("/"))
        resp.set_cookie(config["cookieName"], '', expires=0)
        return resp

if __name__ == "__main__":
    app.run()



#!/usr/bin/env python
"""
Example usage:
    import protocookie
    protocookie.encryptionKey = <encryption key>
    encrypted = encrypt(_userDefinition)
    decrypted = decrypt(encrypted))

with open("./data.json", "r") as fd:
    data = json.load(fd)
"""

from . import protocookie_pb2 as pcookie
import json
from cryptography.fernet import Fernet

encryptionKey = None

def _defineService(obj):
    attrMap = {
            "OWNER"  : pcookie.Service.Role.OWNER,
            "VIEWER" : pcookie.Service.Role.VIEWER,
            "EDITOR" : pcookie.Service.Role.EDITOR,
        }

    service = pcookie.Service()
    service.serviceName = obj["name"]
    service.serviceID = obj["id"]
    list(map(
        service.roles.append,
        list(map(
            attrMap.get,
            obj["perm"]
        ))
    ))

    return service

def _defineUser(obj):
    user = pcookie.User()
    user.username = obj["username"]
    user.id = obj["id"]
    list(map(
        user.services.append, list(map(
            _defineService,
            obj["services"]
            ))
    ))

    return user

def encrypt(obj):
    pbObj = _defineUser(obj)
    cipher_suite = Fernet(encryptionKey)
    return cipher_suite.encrypt(pbObj.SerializeToString())

def decrypt(data):
    cipher_suite = Fernet(encryptionKey)
    return pcookie.User.FromString(cipher_suite.decrypt(data))




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="protocookie",
    version="0.0.1",
    author="Denis Barov",
    author_email="dindin@dindin.ru",
    description="Proof of Concept for storing data in Fernet encrypted protobuffed cookies",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/fordindin/protocookie",
    packages=setuptools.find_packages(),
    package_data={'': ['protocookie.proto', 'config.json']},
    install_requires=[
                    'protobuf',
                    'flask',
                    'cryptography'
                    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: protocookie.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='protocookie.proto',
  package='protocookie',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=_b('\n\x11protocookie.proto\x12\x0bprotocookie\"\x86\x01\n\x07Service\x12\x11\n\tserviceID\x18\x01 \x02(\x05\x12\x13\n\x0bserviceName\x18\x02 \x02(\t\x12(\n\x05roles\x18\x03 \x03(\x0e\x32\x19.protocookie.Service.Role\")\n\x04Role\x12\t\n\x05OWNER\x10\x00\x12\n\n\x06\x45\x44ITOR\x10\x01\x12\n\n\x06VIEWER\x10\x02\"L\n\x04User\x12\x10\n\x08username\x18\x01 \x02(\t\x12\n\n\x02id\x18\x02 \x02(\x05\x12&\n\x08services\x18\x03 \x03(\x0b\x32\x14.protocookie.Service')
)



_SERVICE_ROLE = _descriptor.EnumDescriptor(
  name='Role',
  full_name='protocookie.Service.Role',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OWNER', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='EDITOR', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='VIEWER', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=128,
  serialized_end=169,
)
_sym_db.RegisterEnumDescriptor(_SERVICE_ROLE)


_SERVICE = _descriptor.Descriptor(
  name='Service',
  full_name='protocookie.Service',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='serviceID', full_name='protocookie.Service.serviceID', index=0,
      number=1, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='serviceName', full_name='protocookie.Service.serviceName', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='roles', full_name='protocookie.Service.roles', index=2,
      number=3, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _SERVICE_ROLE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=35,
  serialized_end=169,
)


_USER = _descriptor.Descriptor(
  name='User',
  full_name='protocookie.User',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='username', full_name='protocookie.User.username', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='id', full_name='protocookie.User.id', index=1,
      number=2, type=5, cpp_type=1, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='services', full_name='protocookie.User.services', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=171,
  serialized_end=247,
)

_SERVICE.fields_by_name['roles'].enum_type = _SERVICE_ROLE
_SERVICE_ROLE.containing_type = _SERVICE
_USER.fields_by_name['services'].message_type = _SERVICE
DESCRIPTOR.message_types_by_name['Service'] = _SERVICE
DESCRIPTOR.message_types_by_name['User'] = _USER
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Service = _reflection.GeneratedProtocolMessageType('Service', (_message.Message,), dict(
  DESCRIPTOR = _SERVICE,
  __module__ = 'protocookie_pb2'
  # @@protoc_insertion_point(class_scope:protocookie.Service)
  ))
_sym_db.RegisterMessage(Service)

User = _reflection.GeneratedProtocolMessageType('User', (_message.Message,), dict(
  DESCRIPTOR = _USER,
  __module__ = 'protocookie_pb2'
  # @@protoc_insertion_point(class_scope:protocookie.User)
  ))
_sym_db.RegisterMessage(User)


# @@protoc_insertion_point(module_scope)
